module Simulation

use GlobalVariables
!use SeaTravel, only: Make1D
use ErrorHandling

implicit none

    contains

! RunSimulation -> Used in MAIN
!
! NeoPopGrowth  -> Used by RunSimulation
! NeoPopDispersion  -> Used by RunSimulation
! SaveOutputFile  (currently not used -> Previously used to debug)
! SaveFrontPositionRegion(x0, y0, xf, yf) -> Used by RunSimulation
! SaveSites(iIterations) -> Used by RunSimulation

subroutine RunSimulation(maxIterations, reprRateNeo)
    ! Calls the Neolithic population growth and population dispersal routines at every iteration
    !
    ! Used by Main
    !
    ! Uses NeoPopGrowth, NeoPopDispersion
    ! Uses SaveOutputFile, SaveFrontPosition, SaveSites
    
    implicit none
    integer(4), intent(in) :: maxIterations
    real(8), intent(in) :: reprRateNeo
    integer(4) :: iIterations
    
        do iIterations = 1, maxIterations
                        
            call NeoPopDispersion(iIterations)         
            call NeoPopGrowth(reprRateNeo, iIterations)
            call SaveOutputFile(iIterations)
            call SaveSites(iIterations)

            print *, 'iteration', iIterations
            
        enddo

    call SaveFrontPositionRegion(1, 1, 180, 102)
    
end subroutine RunSimulation


subroutine NeoPopGrowth(reprRateNeo, iIterations)
! Calculate the hunter-gatherer, farmer N and farmer X population density after 
! vertical cultural transmission and population growth process.
! 
! Sets values higher than the carrying capacity (maxNeoPop) at maxNeoPop
! Records front arrival when the farmer population is higher than the threshold
!
! Used by RunSimulation

    implicit none
    real(8), intent(in) :: reprRateNeo
    integer(4), intent(in) :: iIterations
    
    integer :: iNode, j
    real(8) :: localFPop, localPopM, couplesHF, couplesHN, couplesHX
    real(8) :: FPalReprM, FNeoReprM, FPopW, couplesmXwN, couplesmNwX, couplesNX
    real(8) :: couplesNN, couplesXX
    real(8) :: localHGPop
        
    do j = 1, numHabitableNodes  !only habitable nodes can be inhabited
        iNode = habitableNodes(j)
        localFPop = Nodes(iNode)%FPalPopM + Nodes(iNode)%FNeoPopM + Nodes(iNode)%FPalPopW + Nodes(iNode)%FNeoPopW
        
        if (localFPop > 0) then 
            !vertical cultural transmission:
            ! couplesHF = couplesHN + couplesHX = interbreeding * HGPopW * FPopM / (HGPopM + FPopM)
            
            !Total number of men
            localPopM = Nodes(iNode)%HGPopM + Nodes(iNode)%FPalPopM + Nodes(iNode)%FNeoPopM
            
            couplesHF = interbreeding * (Nodes(iNode)%FPalPopM + Nodes(iNode)%FNeoPopM)*Nodes(iNode)%HGPopW / localPopM
            couplesHX = interbreeding * Nodes(iNode)%FPalPopM*Nodes(iNode)%HGPopW / localPopM
            couplesHN = interbreeding * Nodes(iNode)%FNeoPopM*Nodes(iNode)%HGPopW / localPopM

            !HG population after interaction with farmers. They mate and reproduce. They are limited by the number of women
            Nodes(iNode)%HGPopM = (Nodes(iNode)%HGPopW - couplesHF)*reprRateHG
            Nodes(iNode)%HGPopW = (Nodes(iNode)%HGPopW - couplesHF)*reprRateHG
            
            !Single male farmers after the cultural process
            FPalReprM = Nodes(iNode)%FPalPopM - couplesHX
            FNeoReprM = Nodes(iNode)%FNeoPopM - couplesHN

            !Mixed couples among farmers N and X. Limited by the number of men.
            ! couplesmXwN = mX * wN / (wN + wX)
            ! couplesmNwX = mN * wX / (wN + wX)
            
            !farmer women
            FPopW = Nodes(iNode)%FPalPopW + Nodes(iNode)%FNeoPopW
            !Male X, Female N
            couplesmXwN = (FPalReprM * Nodes(iNode)%FNeoPopW) / FPopW
            !Male N, FEemale X
            couplesmNwX = (Nodes(iNode)%FPalPopW * FNeoReprM) / FPopW
            
            couplesNX = couplesmXwN + couplesmNwX
                        
            !Remaining couples of the same kind. Limited by single men
            couplesNN = (FNeoReprM - couplesmNwX)            
            couplesXX = (FPalReprM - couplesmXwN)
            
            !Final farmer population:
            !All couples HF contribute to X 
            Nodes(iNode)%FPalPopM = reprRateNeo*(couplesXX + couplesmNwX + couplesHF)
            Nodes(iNode)%FPalPopW = Nodes(iNode)%FPalPopM !50% of the children are boy and 50% girls
            
            Nodes(iNode)%FNeoPopM = reprRateNeo*(couplesNN + couplesmXwN)
            Nodes(iNode)%FNeoPopW = Nodes(iNode)%FNeoPopM !50% of the children are boy and 50% girls
        end if
    enddo

    !maximum control population
    do j = 1, numHabitableNodes
        iNode = habitableNodes(j)
        localFPop = Nodes(iNode)%FPalPopM + Nodes(iNode)%FNeoPopM + Nodes(iNode)%FPalPopW + Nodes(iNode)%FNeoPopW
 
        if (localFPop > 0) then
            if (localFPop >= maxFPop) then
                Nodes(iNode)%FPalPopM = Nodes(iNode)%FPalPopM * maxFPop / localFPop
                Nodes(iNode)%FNeoPopM = Nodes(iNode)%FNeoPopM * maxFPop / localFPop

                Nodes(iNode)%FPalPopW = Nodes(iNode)%FPalPopW * maxFPop / localFPop
                Nodes(iNode)%FNeoPopW = Nodes(iNode)%FNeoPopW * maxFPop / localFPop
            end if
            if (localFPop > threshold) then
                if (Nodes(iNode)%frontArrival == 500) Nodes(iNode)%frontArrival = iIterations
            endif
        end if
        
        if ((Nodes(iNode)%HGPopM + Nodes(iNode)%HGPopW) > 0.) then
            if ((Nodes(iNode)%HGPopM + Nodes(iNode)%HGPopW) > maxHGPop) then
                localHGPop = Nodes(iNode)%HGPopM + Nodes(iNode)%HGPopW
                Nodes(iNode)%HGPopM = Nodes(iNode)%HGPopM * maxHGPop / localHGPop
                Nodes(iNode)%HGPopW = Nodes(iNode)%HGPopW * maxHGPop / localHGPop
            endif
        endif

    end do
end subroutine NeoPopGrowth

subroutine NeoPopDispersion(iIterations)
    ! Compute the farmer populations N and X after dispersion
    ! 
    ! Sets values higher than the carrying capacity (maxNeoPop) at maxNeoPop
    ! Records front arrival when the farmer population is higher than the threshold
    !
    ! Used by RunSimulation
    implicit none
    integer(4), intent(in) :: iIterations !iteration number
    integer(4) :: i, j, iNode, numDest, destination
    real(8) :: destPop, destNeoPopM, destPalPopM, destNeoPopW, destPalPopW
    real(8) :: farmerPop, initialFNeoM, initialFPalM, initialFNeoW, initialFPalW
    real(8) :: dens, densNeoM, densPalM, densNeoW, densPalW
    
    !Dispersion of farmers (N and X genetics)
    do i = 1, numHabitableNodes
        iNode = habitableNodes(i)
        farmerPop = Nodes(iNode)%FNeoPopM + Nodes(iNode)%FPalPopM + Nodes(iNode)%FNeoPopW + Nodes(iNode)%FPalPopW
        
        initialFNeoM = Nodes(iNode)%FNeoPopM
        initialFPalM = Nodes(iNode)%FPalPopM
        
        initialFNeoW = Nodes(iNode)%FNeoPopW
        initialFPalW = Nodes(iNode)%FPalPopW
        
        if (farmerPop > 0) then
            !SUbpopulations initially at the origin. Used to compute the fraction that moves
            densNeoM = initialFNeoM
            densPalM = initialFPalM
            
            densNeoW = initialFNeoW
            densPalW = initialFPalW
            
            !Dispersion destinations
            numDest = Nodes(iNode)%numDestinations
            
            do j = 1, numDest
                destination = Nodes(iNode)%destinations(j)
                
                !Population that moves to destination
                destNeoPopM = densNeoM*(Nodes(iNode)%destinationWeight(j))
                destPalPopM = densPalM*(Nodes(iNode)%destinationWeight(j))

                destNeoPopW = densNeoW*(Nodes(iNode)%destinationWeight(j))
                destPalPopW = densPalW*(Nodes(iNode)%destinationWeight(j))
                
                !New total population at the destination after migration
                Nodes(destination)%FNeoPop2M = Nodes(destination)%FNeoPop2M + destNeoPopM 
                Nodes(destination)%FPalPop2M = Nodes(destination)%FPalPop2M + destPalPopM 

                Nodes(destination)%FNeoPop2W = Nodes(destination)%FNeoPop2W + destNeoPopW 
                Nodes(destination)%FPalPop2W = Nodes(destination)%FPalPop2W + destPalPopW 

                ! Population that has not yet moved
                initialFNeoM = initialFNeoM - destNeoPopM 
                initialFPalM = initialFPalM - destPalPopM 

                initialFNeoW = initialFNeoW - destNeoPopW 
                initialFPalW = initialFPalW - destPalPopW 
                
            enddo    
            
            ! Population that stays at the origin node
            Nodes(iNode)%FNeoPop2M = Nodes(iNode)%FNeoPop2M + initialFNeoM
            Nodes(iNode)%FPalPop2M = Nodes(iNode)%FPalPop2M + initialFPalM
            
            Nodes(iNode)%FNeoPop2W = Nodes(iNode)%FNeoPop2W + initialFNeoW
            Nodes(iNode)%FPalPop2W = Nodes(iNode)%FPalPop2W + initialFPalW
                        
        endif
    enddo

    ! Set final population after dispersion.
    ! Record arrival if farmer population > threshold
    do i = 1, numHabitableNodes
        iNode = habitableNodes(i)
        
        !Set new population after dispersal
        Nodes(iNode)%FNeoPopM = Nodes(iNode)%FNeoPop2M
        Nodes(iNode)%FPalPopM = Nodes(iNode)%FPalPop2M
        
        Nodes(iNode)%FNeoPopW = Nodes(iNode)%FNeoPop2W
        Nodes(iNode)%FPalPopW = Nodes(iNode)%FPalPop2W
        
        
        !Record arrival
        farmerPop = Nodes(iNode)%FNeoPopM + Nodes(iNode)%FPalPopM + Nodes(iNode)%FNeoPopW + Nodes(iNode)%FPalPopW
        
        if (farmerPop > threshold) then
            if (Nodes(iNode)%frontArrival == 500) Nodes(iNode)%frontArrival = iIterations
        endif
        
        !Set computing variables to 0
        Nodes(iNode)%FNeoPop2M = 0.
        Nodes(iNode)%FPalPop2M = 0.

        Nodes(iNode)%FNeoPop2W = 0.
        Nodes(iNode)%FPalPop2W = 0.
    enddo 
    
end subroutine NeoPopDispersion

subroutine SaveOutputFile(iIteracions)
    ! Records each subpopulation and the fraction of haplogroup K at each node and interation
    ! Saves a file per iteration
    !
    !Used by RunSimulation
    implicit none
    integer, intent(in) :: iIteracions
    integer :: iNode
    real(8) :: percentatge
    character(len=3) :: str
    character(len=20) :: name
    
    !Name the file as the current iteration
    write (str, '(i3)') iIteracions
    name = 'results'//str//'gen.txt'

    !Save the information
    open (13, file=name, status="unknown")
    write (13,fmt="(a5, 10a15)") 'Node','Longitude', 'Latitude','HG_Men_Pop', 'HG_Women_Pop', 'X_Men_Pop', 'X_Women_Pop', 'F_Men_Pop', 'F_Women_Pop', 'percentage'
    
    do iNode = 1, numNodes
        !%K at the node
        percentatge = 100. *(Nodes(iNode)%FNeoPopM + Nodes(iNode)%FNeoPopW)/(Nodes(iNode)%FNeoPopM + Nodes(iNode)%FNeoPopW+ Nodes(iNode)%FPalPopM + Nodes(iNode)%FPalPopW)
        ! Save results
        write (13, fmt="(i5,2f15.8, 6i15, f15.8)") Nodes(iNode)%node_id, Nodes(iNode)%longitude, Nodes(iNode)%latitude, &
            & nint(Nodes(iNode)%HGPopM), nint(Nodes(iNode)%HGPopM), nint(Nodes(iNode)%FPalPopM), nint(Nodes(iNode)%FPalPopW), &
            & nint(Nodes(iNode)%FNeoPopM), nint(Nodes(iNode)%FNeoPopW), percentatge
    enddo

    close(13)
end subroutine SaveOutputFile

subroutine SaveFrontPositionRegion(x0, y0, xf, yf)
    ! Save the arrival time at each node within a certain region of the grid
    !
    !Used by RunSimulations
    implicit none
    integer(4), intent(in) :: x0, y0, xf, yf
    integer(4) :: i, j, iNode
        
    open (14, file = "front.dat", status = "unknown")
    
    write (14, fmt="(a5, 2a15, 3a8)") "Node", "Longitude", "Latitude", "X (km)", "Y (km)", "Arriv"
    do j = y0, yf
        do i = x0, xf
            iNode = nodesGrid(i,j)
            !if (Nodes(iNode)%frontArrival < 500) then  !Save only location that have been reached
            if (Nodes(iNode)%node_type /= 0) then   !Save locations not on the sea
                
                !write (14, fmt="(i5,2f15.8,i5,4f15.8)") Nodes(iNode)%node_id, Nodes(iNode)%longitude, Nodes(iNode)%latitude, Nodes(iNode)%frontArrival
                !&, Nodes(iNode)%FPalPopM, Nodes(iNode)%FPalPopW, Nodes(iNode)%FNeoPopM, Nodes(iNode)%FNeoPopW
                write (14, fmt="(i5,2f15.8,3i8)") Nodes(iNode)%node_id, Nodes(iNode)%longitude, Nodes(iNode)%latitude, &
                    50*Nodes(iNode)%x, 50*Nodes(iNode)%y, Nodes(iNode)%frontArrival
            endif
        enddo    
    enddo
    close (14)
endsubroutine

subroutine SaveSites(iIterations)
    ! Save the population and %K at the locations and date of the genetic data
    ! considering that the simulation starts at 8233 BCE
    !
    ! Used by RunSimulation
    integer, intent(in) :: iIterations
    integer :: node, file_err
    real(8) :: fraction, FNeoPop, FPalPop, HGPop


    select case (iIterations)
    case (30)
        open (55, file="Kfraction.txt", status="unknown", iostat=file_err)
        call OpenFileError(file_err, "There has been an error while opening 'Kfraction.txt' file.")
        write(55,fmt="(a8,a15, 4a8)") "Node", "Region", "HG pop", "X pop", "N pop", "Kfrac"
        node = 5695
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "Syria", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
    case (62)
        node = 7662
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "Anatolia", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
    case (79)
        node = 10347
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "Hung-Croat", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
    case (92)
        node = 10318
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "SpainCard", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
    case (95)
        node = 10662
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "Portugal", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
    case (97)
        node = 13040
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "EastGerm", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
        node = 13220
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "WestGerm", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
    case (102)
        node = 10854
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "Navarre", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
    case (169)
        node = 15387
        FNeoPop = Nodes(node)%FNeoPopM + Nodes(node)%FNeoPopW
        FPalPop = Nodes(node)%FPalPopM + Nodes(node)%FPalPopW
        HGPop = Nodes(node)%HGPopM + Nodes(node)%HGPopW
        fraction = 100 * FNeoPop / (FPalPop + FNeoPop)
        write(55, fmt="(i8, a15, 3i8, f8.4)") node, "Sweden", nint(HGPop), nint(FPalPop), nint(FNeoPop), fraction
        close (55)
    end select
end subroutine SaveSites


end module Simulation